<?php

/* --------------------------------------------------------------
  CustomerGroup.inc.php 2017-10-02
  Gambio GmbH
  http://www.gambio.de
  Copyright (c) 2017 Gambio GmbH
  Released under the GNU General Public License (Version 2)
  [http://www.gnu.org/licenses/gpl-2.0.html]
  --------------------------------------------------------------
*/

/**
 * Class QuantityUnitCollection
 *
 * @category   System
 * @package    CustomerGroup
 * @subpackage Entities
 */
class CustomerGroup implements CustomerGroupInterface
{
	/**
	 * Customer group id.
	 *
	 * @var int
	 */
	protected $id = 0;
	
	/**
	 * @var \EditableKeyValueCollection
	 */
	protected $names;
	
	/**
	 * @var CustomerGroupSettingsInterface
	 */
	protected $settings;
	
	/**
	 * @var CustomerGroupConfigurationsInterface
	 */
	protected $configurations;
	
	/**
	 * @var CustomerGroupRepositoryInterface
	 */
	protected $repository;
	
	
	/**
	 * CustomerGroup constructor.
	 *
	 * @param \CustomerGroupRepositoryInterface $repository
	 * @param \EditableKeyValueCollection       $names
	 */
	public function __construct(CustomerGroupRepositoryInterface $repository, EditableKeyValueCollection $names)
	{
		$this->repository     = $repository;
		$defaultBool          = new BoolType(false);
		$this->names          = $names;
		$this->configurations = MainFactory::create('CustomerGroupConfigurations', new DecimalType(0.00),
		                                            new DecimalType(0.00), new DecimalType(0.00), new DecimalType(0.00),
		                                            [], []);
		$this->settings       = MainFactory::create('CustomerGroupSettings', $defaultBool, $defaultBool, $defaultBool,
		                                            $defaultBool, $defaultBool, $defaultBool, $defaultBool,
		                                            $defaultBool, $defaultBool, $defaultBool, $defaultBool);
	}
	
	
	/**
	 * Saves customer group in database.
	 *
	 * @return $this|\CustomerGroupInterface Same instance for chained method calls.
	 */
	public function store()
	{
		$this->repository->store($this);
		
		return $this;
	}
	
	
	/**
	 * Updates customer group in database.
	 *
	 * @return $this|\CustomerGroupInterface Same instance for chained method calls.
	 */
	public function update()
	{
		$this->repository->update($this);
		
		return $this;
	}
	
	
	/**
	 * Deletes customer group from database.
	 *
	 * @return $this|\CustomerGroupInterface Same instance for chained method calls.
	 */
	public function delete()
	{
		$this->repository->delete($this);
		
		return $this;
	}
	
	
	/**
	 * Returns the customer group id.
	 *
	 * @return int Customer group id.
	 */
	public function getId()
	{
		return $this->id;
	}
	
	
	/**
	 * Sets the customer group id.
	 *
	 * @param \IntType $id The customer group id to be set.
	 *
	 * @return $this|\CustomerGroupInterface Same instance for chained method calls.
	 */
	public function setId(IntType $id)
	{
		$this->id = $id->asInt();
		
		return $this;
	}
	
	
	/**
	 * Returns the name.
	 *
	 * @param \LanguageCode $languageCode Language code for language specific name value.
	 *
	 * @return string Language specific name value.
	 */
	public function getName(LanguageCode $languageCode)
	{
		return $this->names->getValue($languageCode->asString());
	}
	
	
	/**
	 * Sets the name.
	 *
	 * @param \StringType   $name         Name.
	 * @param \LanguageCode $languageCode Language code for language specific name value.
	 *
	 * @return $this|\CustomerGroupInterface Same instance for chained method calls.
	 */
	public function setName(StringType $name, LanguageCode $languageCode)
	{
		$this->names->setValue($languageCode->asString(), $name->asString());
		
		return $this;
	}
	
	
	/**
	 * Returns all language specific name.
	 *
	 * @return array Language specific name value.
	 */
	public function getNames()
	{
		return $this->names->getArray();
	}
	
	
	/**
	 * Returns the customer group settings.
	 *
	 * @return \CustomerGroupSettingsInterface
	 */
	public function getSettings()
	{
		return $this->settings;
	}
	
	
	/**
	 * Sets the customer group settings.
	 *
	 * @param \CustomerGroupSettingsInterface $settings Customer group settings.
	 *
	 * @return $this|\CustomerGroupInterface Same instance for chained method calls.
	 */
	public function setSettings(CustomerGroupSettingsInterface $settings)
	{
		$this->settings = $settings;
		
		return $this;
	}
	
	
	/**
	 * Returns the customer group configurations.
	 *
	 * @return \CustomerGroupConfigurationsInterface
	 */
	public function getConfigurations()
	{
		return $this->configurations;
	}
	
	
	/**
	 * Sets the customer group configurations.
	 *
	 * @param \CustomerGroupConfigurationsInterface $configurations Customer group configurations.
	 *
	 * @return $this|\CustomerGroupInterface Same instance for chained method calls.
	 */
	public function setConfigurations(CustomerGroupConfigurationsInterface $configurations)
	{
		$this->configurations = $configurations;
		
		return $this;
	}
}